;Schaltkreis spezifische Einstellungen ...
.include	"tn2313def.inc"
;
;hier den Debugmodus einschalten, dann werden die Empfangenen Codes
;ber die Serielle Schnittstelle ausgegeben ...
;AVR luft mit 8MHz

.equ	DEBUG = 1

;Hier Anschluss fr TSOP7000 anpassen
;IO-Bitdefinitionen
.equ	IR_IN_Bit	=	2
.equ	IR_IN_Port	=	PIND

;IO-Portmasken und Startwerte
.equ	PortB_DIR	=	0b00000000
.equ	PortB_DAT	=	0b00000000
.equ	PortD_DIR	=	0b00000000
.equ	PortD_DAT	=	0b00000000

;BeoEmpf. Werte		** Timer luft mit 1MHz =^= 0.256 ms
;			** pro berlauf - MaxZeit = 65.28ms
.equ	TK		=	4	;rund 4 Takte pro ms
.equ	TimerMax	=	68	;17*TK
.equ	BeoMaxDiff	=	3	;0.75*TK
.equ	BeoBit0Time	=	12	;3*TK
.equ	BeoBit1Time	=	24	;6*TK
.equ	BeoBit2Time	=	36	;9*TK

;Registerdefinitionen
.def	Reg0		=	r0
.def	TempL0		=	r1
.def	TempL1		=	r2
.def	TempL2		=	r3
.def	ISNULL		=	r9
.def	SRegSave 	=	r14

.def	TempU0		=	r16
.def	TempU1		=	r17
.def	TempU2		=	r18
.def	TempU3		=	r19
.def	Timer		=	r24
.
;Macrodefionen
.macro	PushZ	;ZH und ZL auf Stack
	push	ZH
	push	ZL
.endmacro
.macro	PopZ
	pop	ZL
	pop	ZH
.endmacro
.macro	PushX	;XH und XL auf Stack
	push	XH
	push	XL
.endmacro
.macro	PopX
	pop	XL
	pop	XH
.endmacro
.macro	adi
	subi	@0,256-@1
.endmacro
.macro	mov_w	;16-Bit Reg. kopieren
	mov	@0L,@1L
	mov	@0H,@1H
.endmacro
.macro	ldiw_m	;16-Bit Adresse aus FLASH laden
	ldi	@0L,low(@1*2)
	ldi	@0H,high(@1*2)
.endmacro
.macro	ldiw	;16-Bit Adresse aus EEProm / Ram laden
	ldi	@0L,low(@1)
	ldi	@0H,high(@1)
.endmacro
.macro	stsw	;16-Bit Wert in Ram-speichern
	sts	@0,@1L
	sts	@0+1,@1H
.endmacro 
.macro	ldsw	;16-Bit Wert aus Ram-speichern laden
	lds	@0L,@1
	lds	@0H,@1+1
.endmacro 
.macro	EE_Read
	sbi	EECR,0
.endmacro

RESET:
I_Reset:	rjmp	Start
I_Int0:		reti
I_Int1:		reti
I_Timer1_Capt:	reti
I_Timer1_CompA:	reti
I_Timer1_OV:	reti
I_Timer0_OV:	rjmp	Timer0_OV
I_UART_Rx:	reti
I_UART_UDRE:	reti
I_UART_Tx:	reti
I_ANA_CMP:	reti
I_PCINT:	reti
I_TIMER1_CompB:	reti
I_TIMER0_CompA:	reti
I_TIMER0_CompB:	reti
I_USI_STR:	reti
I_USI_OV:	reti
I_EERdy:	reti
I_WDT_OV:	reti


BeoBitTimes:	;* 3;3;15;3;6;6;6;xxx;12
		.db	56,56,8,56,44,44,44,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0
;***************************************************************
;* Sendet Byte (TempU1) zur Schnittstelle
Send_RS232:
Sser_wl0:	sbis	USR,UDRE
		rjmp	Sser_wl0	;warten bis kein Zeichen
		out	UDR,TempU1
		ret
;***************************************************************
;* Aufruf alle 0.256 ms
Timer0_OV:	;wird per Timer0_OV_Int. ausgelst
		in	SRegSave,SReg
		dec	Timer
		brne	Timer0_End
		clr	TempL0		;wenn 0 dann
		out	TCCR0,TempL0	;Timer aus		
Timer0_End:	out	SReg,SRegSave
		reti

;***************************************************************
;* wertet das vom IR-Empfnger kommende Signal aus
;* Erfolgreich bei T=1
;* liefert fr alle Gerte <> Licht T=0
;* Taste wird in 'KeyCode' im RAM gespeichert
ReadBeo:	;L signal an IRIN wurde erkannt !
		clr	TempU1
		out	TCNT0,TempU1	;Counter0 = 0
		ldi	TempU1,2
		out	TCCR0,TempU1	;ZeitMessung Starten
		ldi	Timer,TimerMax	;Max Time

		ldi	TempU1,21	;Anzahl der Bits
		PushZ
		ldiw	X,BeoReadBits
		ldiw_m	Z,BeoBitTimes
Beo_Next_Bit:	
Beo_Wait_H:	;auf H-Warten und in der (L)Zeit messen
		tst	Timer
		breq	Beo_Error
		sbis	IR_IN_PORT,IR_IN_Bit	;auf H warten
		rjmp	Beo_Wait_H
		;jetzt H-Zeit (Bitlnge) messen
Beo_Time_Loop:	tst	Timer		;Zeitberlauf ?? (Timer stoppt automatisch !!)
		breq	Beo_Error
		sbic	IR_IN_PORT,IR_IN_Bit	;messen bis L
		rjmp	Beo_Time_Loop
	;H-Zeit zu Ende ...
		mov	TempU3,Timer	;Timer-Wert sichern
		clr	TempL1
		out	TCNT0,TempL1	;Counter0 = 0
		ldi	Timer,TimerMax	;maximalwert
		lpm	;(Z) -> r0
		ld	TempL1,Z+	;***** Y=Y+1 *****
		mov	TempL1,r0	;vergleichswert laden
		tst	TempL1		;=0 ,dann Einzelbitauswertung
		breq	Beo_Bitauswert
		sub	TempU3,TempL1	;Zeitdifferenz
		sbrc	TempU3,7		;negativ ??
		neg	TempU3
		cpi	TempU3,BeoMaxDiff+1
		brcc	Beo_Error
Beo_Bit_Ok:	dec	TempU1
		brne	Beo_Next_Bit
		clr	TempL1
		out	TCCR0,TempL1	;Zeitmessung stoppen		
		rjmp	Beo_Read_Ok
Beo_Error:	clt
		PopZ
					;T=0 -> FEHLER
.ifdef DEBUG
		ldi	TempU1,'E'
		rcall	Send_RS232
		mov	TempU1,Timer
		rcall	spl_var
		rcall	Send_RS232
		mov	TempU1,TempU2
		rcall	Send_RS232
.endif	
		ret
Beo_Bitauswert:	;TempU3 enthlt Timerwert
		clr	TempL1
Beo_Bit_T0:	ldi	TempU2,TimerMax-BeoBit0Time+BeoMaxDiff
		cp	TempU2,TempU3	
		brcs	Beo_Error	;Zeit kleiner als kleinste BitZeit
		cpi	TempU3,TimerMax-BeoBit0Time-BeoMaxDiff
		brcs	Beo_Bit_T1
		st	X+,TempL1	;Bit=0
		rjmp	Beo_Bit_Ok
Beo_Bit_T1:	inc	TempL1
		ldi	TempU2,TimerMax-BeoBit1Time+BeoMaxDiff
		cp	TempU2,TempU3
		brcs	Beo_Error
		cpi	TempU3,TimerMax-BeoBit1Time-BeoMaxDiff
		brcs	Beo_Bit_T2
		st	X+,TempL1	;Bit=1
		rjmp	Beo_Bit_Ok
Beo_Bit_T2:	inc	TempL1
		ldi	TempU2,TimerMax-BeoBit2Time+BeoMaxDiff
		cp	TempU2,TempU3
		brcs	Beo_Error
		cpi	TempU3,TimerMax-BeoBit2Time-BeoMaxDiff
		brcs	Beo_Error
		st	X+,TempL1	;Bit=2
		rjmp	Beo_Bit_Ok
Beo_Read_Ok:
.ifdef DEBUG
		ldi	TempU1,'='
		rcall	Send_RS232
.endif
	;Gertecode ermitteln
		ldiw	X,BeoReadBits
		ldi	TempU1,6
		clr	YL
		clr	YH
		clr	TempL2
Beo_Dev_Loop:	ld	TempL1,X+
		mov_w	Z,Y
		add	YL,YL
		adc	YH,YH		;CY-ADD *2
		add	YL,ZL
		adc	YH,ZH		;CY-ADD *3
		add	YL,TempL1
		adc	YH,TempL2	;CY-ADD
		dec	TempU1
		brne	Beo_Dev_Loop
		stsw	DevCode,Y	;Gertecode in Y
	;Tastencode ermitteln
		ldi	TempU1,7							
		clr	YL
		clr	YH
Beo_Key_Loop:	ld	TempL1,X+
		mov_w	Z,Y
		add	YL,YL
		adc	YH,YH		;CY-ADD *2
		add	YL,ZL
		adc	YH,ZH		;CY-ADD *3
		add	YL,TempL1
		adc	YH,TempL2	;CY-ADD
		dec	TempU1
		brne	Beo_Key_Loop
		stsw	KeyCode,Y	;Tastencode in Y
		PopZ
		set	
		ret

;###############################################################
;###############################################################
;###############################################################

;**** H A U P T P R O G R A M M *****
;************************************
					
Start:		clr	ISNULL
	;PortB Init
		ldi	TempU1,PortB_DAT
		out	PortB,TempU1
		ldi	TempU1,PortB_DIR
		out	DDRB,TempU1
	;PortD Init
		ldi	TempU1,PortD_DAT
		out	PortD,TempU1
		ldi	TempU1,PortD_DIR
		out	DDRD,TempU1
	;Stakepointer setzen
		ldi	TempU1,Low(RAMEND)
		out	SPL,TempU1
	;Timer 0 konfiguriern
		ldi	TempU1,2
		out	TCCR0,TempU1		;ZeitMessung Starten f/8
		ldi	TempU1,0b00000010	;Timer0 OV_INT   >EIN
		out	TIMSK,TempU1
	;UART init
		ldi	TempU1,25		;19200 Baud
		out	UBRRL,TempU1
		ldi	TempU1,0b00001000	;Sender  >EIN
		out	UCSRB,TempU1
		ldi	TempU1,0b00000000
		out	UCSRA,TempU1

		sei
;Beginn der  Haupschleife
.ifdef DEBUG
		rcall	Hallo
.endif


HLoop:		clt
		sbis	IR_IN_PORT,IR_IN_Bit		;IR Signal ????
		rcall	ReadBeo
		brtc	HLoop		;nein
		
.ifdef DEBUG
		rcall	show_taste
.endif

;Tastenverteiler	
	;@ENRICO ...
	;
	;so wenn das Prog. hier ankommt wurde eine gltige Taste empfangen
	;die Codes fr Gert und Taste kannst du mit aus dem RAM nehmen
	;(DevCode und KeyCode) und auswerten





key_end:	
.ifdef DEBUG
		rcall	send_lf
.endif
		rjmp	HLoop


.ifdef DEBUG
		.include	"debug_routines.asm"
.endif



;#####################################################################
;Daten im FLASH-Speicher
DimmTable:	
	.dw	16,16,26,36,47,58,70,82,95,108,122,136,151,166,182
	.dw	198,215,232,250,268,286,305,324,344,364,385,406,428,450,450




.dseg
.org	$60
BeoReadBits:	.byte	14
KeyCode:	.byte	2
DevCode:	.byte	2
